var Global_DP_Fishing = {
    dir: 'res/game/fishing/',
    layer: null,
    space: null,
    fish_rod: null,
    fish_hook: null,
    coin_box: null,
    score: 0,
    gameOver: false,
};

var DP_FishingLayer = cc.Layer.extend({
    timeRemain: 0,
    stepTime: 1 / 60.0,
    difficulty: 0,
    ctor: function () {
        this._super();
        this.x = cc.winSize.width / 2;
        this.y = Global_DP.sinkY;
        Global_DP_Fishing.layer = this;

        this.addRes();
        this.addFish();
        this.loadPhysicsSpace();
        // this.loadPhysicsSpaceDebug();
        this.addFishRod();
        this.addTimeClock();
        this.addCoinBox();

        zAddBackButtonBegin(this);

        this.scheduleUpdate();
    },
    onEnter: function () {
        this._super();

        Global_DP_Fishing.gameOver = false;
        zPauseTargetEvent(this, true);
        this.countDown = function () {
            var coin_box = Global_DP_Fishing.coin_box;
            this.count_down.count();
            coin_box.setCoinNumber(zGetLocalCoinNumber());
            zResumeTargetEvent(this, true);
            zPauseTargetEvent(Global_DP_Fishing.fish_rod, true);
        };
        zAddInsertCoinsLayer(this, this.countDown);

    },
    update: function (dt) {
        this.timeRemain += dt;
        while (this.timeRemain - this.stepTime >= 0) {
            this.space.step(this.stepTime);
            this.timeRemain -= this.stepTime;
        }
        var fish_rod = Global_DP_Fishing.fish_rod;
        var fish_hook = Global_DP_Fishing.fish_hook;
        var fish_line = this.lineSprite;
        var p1 = fish_rod.getPosition();
        var p2 = fish_hook.getPosition();
        var dist = cc.pDistance(p1, p2);
        var angle = Math.acos((p1.y - p2.y) / dist) / Math.PI * 180;
        if (p1.x - p2.x < 0) {
            angle = 180 - angle;
        }
        fish_line.setPosition(cc.pMidpoint(p1, p2));
        fish_line.setScaleY(dist / fish_line.height);
        if (!isNaN(angle)) {
            fish_line.setRotation(angle);
        }
    },
    addRes: function () {
        var arrName = ['bg_hd', 'fish_basket', 'fish_machine'];
        var res = zAddRes(Global_DP_Fishing.dir, arrName, this);
        this.bg_hd = res[0];
        this.bg_hd.setAnchorPoint(0.5, 0);
        this.fish_basket = res[1];
        this.fish_basket.setPosition(this.convertToNodeSpace(cc.p(cc.winSize.width, 0)));
        var posCenter = this.convertToNodeSpace(cc.p(cc.winSize.width / 2, cc.winSize.height / 2));
        this.fish_machine = res[2];
        this.fish_machine.setPosition(posCenter);
        this.fish_machine.fish_pos = [
            cc.p(351.77, 622.29), cc.p(579.61, 541.46), cc.p(166.38, 468.20), cc.p(463.63, 173.53),
            cc.p(606.94, 342.43), cc.p(207.27, 230.22), cc.p(390.61, 391.99)
        ];

        // this.bg_hd.setVisible(false);
        // this.fish_machine.setVisible(false);

        // var node = new cc.Node();
        // node.setContentSize(cc.size(135, 135));
        // node.setAnchorPoint(0.5, 0.5);
        // this.fish_machine.addChild(node);
        // zShowContentSize(node, cc.size(135, 135));
        // zAddTouchLogPosEvent(node);
    },
    loadPhysicsSpace: function () {
        this.space = new cp.Space();
        this.space.gravity = cp.v(0, -1000);
        this.space.sleepTimeThreshold = 0.5;
        this.space.damping = 0.3;
        Global_DP_Fishing.space = this.space;
    },
    loadPhysicsSpaceDebug: function () {
        var node = new cc.PhysicsDebugNode(this.space);
        this.addChild(node, Number.MAX_VALUE);
    },
    addFishRod: function () {
        var space = this.space;
        var dir = Global_DP_Fishing.dir;

        // 鱼竿
        var rodSide = 1;
        var rodBody = new cp.StaticBody();
        var rodShape = new cp.BoxShape(rodBody, rodSide, rodSide);
        var rodSprite = new cc.PhysicsSprite(zGetFullPathPNG(dir, 'fishing_rod'));
        rodBody.setPos(cp.v(cc.winSize.width * 0.3, cc.winSize.height * 0.8));
        rodSprite.setBody(rodBody);
        rodSprite.setAnchorPoint(11 / rodSprite.width, 307 / rodSprite.height);
        space.addShape(rodShape);
        this.addChild(rodSprite, 10);

        // 鱼钩
        var hookMess = 100;
        var hookSide = 15;
        var hookBody = new cp.Body(hookMess, cp.momentForBox(hookMess, hookSide, hookSide));
        var hookShape = new cp.BoxShape(hookBody, hookSide, hookSide);
        var hookSprite = new cc.PhysicsSprite(zGetFullPathPNG(dir, 'fishing_hook'));
        hookBody.setPos(cp.v.sub(rodBody.getPos(), cp.v(0, 150)));
        hookBody.setAngle(-45);
        hookSprite.setBody(hookBody);
        hookSprite.setAnchorPoint(0.8, 0.8);
        space.addBody(hookBody);
        space.addShape(hookShape);
        this.addChild(hookSprite, 10);

        // 鱼坠
        // var dropMess = 150;
        // var dropSide = 5;
        // var dropBody = new cp.Body(dropMess, cp.momentForBox(dropMess, dropSide, dropSide));
        // var dropShape = new cp.BoxShape(dropBody, dropSide, dropSide);
        // dropBody.setPos(cp.v.sub(hookBody.getPos(), cp.v(0, 150)));
        // space.addBody(dropBody);
        // space.addShape(dropShape);

        // 约束
        var lenMin = 0;
        var lenMax = 200;
        var slideRodToHook = new cp.SlideJoint(rodBody, hookBody, cp.v(0, 0), cp.v(hookSide / 2, hookSide / 2), lenMin, lenMax);
        // var slideHookToDrop = new cp.SlideJoint(hookBody, dropBody, cp.v(-hookSide / 2, -hookSide / 2), cp.v(0, 0), lenMin, lenMax - 100);
        space.addConstraint(slideRodToHook);
        // space.addConstraint(slideHookToDrop);

        var onTB = function (touch, target) {
            target.stopAllActions();
        };
        var onTM = function (touch, target) {
            var delta = touch.getDelta();
            var space = Global_DP_Fishing.space;
            if (target.x + delta.x < -400) {
                target.x = -400;
            } else if (target.x + delta.y > 400) {
                target.x = 400;
            } else {
                target.x += delta.x;
                target.y += delta.y;
            }
            // zLogPos(cp.v(target.x, target.y));
            hookBody.activate();
            rodBody.setPos(cp.v(target.x, target.y));
            space.reindexStatic();
        };
        var listener = zNewEventListenerCreate(onTB, onTM, null, null);
        zSetNodeEvent(listener, rodSprite);

        // 鱼线
        this.lineSprite = zNewSpriteCreate(dir, 'fishing_line', this);
        this.lineSprite.setLocalZOrder(5);
        Global_DP_Fishing.fish_rod = rodSprite;
        Global_DP_Fishing.fish_hook = hookSprite;

        // 鱼篓边界
        var boundShapeR = new cp.SegmentShape(new cp.StaticBody(), cp.v(cc.winSize.width / 2 + 50, 0), cp.v(cc.winSize.width / 2 + 50, 300), 2);
        var boundShapeD = new cp.SegmentShape(new cp.StaticBody(), cp.v(cc.winSize.width / 2 - 300, 0 - 50), cp.v(cc.winSize.width / 2, 0 - 50), 2);
        this.space.addShape(boundShapeR);
        this.space.addShape(boundShapeD);
    },
    addFish: function () {
        var self = this;
        var dir = Global_DP_Fishing.dir;
        var fish_pos = this.fish_machine.fish_pos;
        var fishCount = fish_pos.length;
        for (var i = 0; i < fishCount; ++i) {
            var idt = (i < 6 ? i : Math.floor(Math.random() * 6));
            // var fish = zNewSpriteCreate(dir, 'fish' + idt, this.fish_machine);
            var fishNode = new cc.Node();
            var fishSpine = zNewSpineAnimationCreate(dir + 'spine/', 'skeleton', fishNode);
            var fishSpineOffsetY = 17;
            var angle = Math.random() * 360;
            // var angle = 90 * i;
            var rad = angle / 180 * Math.PI;
            this.fish_machine.addChild(fishNode);
            fishNode.setPosition(fish_pos[i].x - fishSpineOffsetY * Math.sin(rad), fish_pos[i].y - fishSpineOffsetY * Math.cos(rad));
            fishNode.setContentSize(cc.size(1, 150));
            fishNode.setRotation(angle);
            fishSpine.setAnimation(0, idt + 'zhang', false);
            fishSpine.setTimeScale(1 + (Math.random() - 0.5) / 2);
            fishSpine.setPosition(0, fishSpineOffsetY);
            // fishSpine.setVisible(false);
            // zShowContentSize(fishNode, cc.size(1, 150));
            (function f(fishNode, fishSpine, idt) {
                fishSpine.setCompleteListener((trackEntry) => {
                    var hook = Global_DP_Fishing.fish_hook;
                    var name = trackEntry.animation ? trackEntry.animation.name : '';
                    // zLog('name : ', name);
                    if (Global_DP_Fishing.gameOver) {
                        return;
                    }
                    if (cc.pDistance(zGetWorldPosition(hook), zGetWorldPosition(fishNode)) < 100 && hook.getChildren().length <= 0) {
                        var rod = Global_DP_Fishing.fish_rod;
                        zChangeParent(fishNode, hook);
                        ddSound.playEffect('game_fishing_bite_hook');
                        fishSpine.setAnimation(0, idt + 'he', false);
                        fishNode.unscheduleAllCallbacks();
                        fishNode.setPosition(hook.width * 0.8, hook.height * 0.8);
                        fishNode.setRotation(-135);
                        hook.setLocalZOrder(0);
                        zPauseTargetEvent(rod, true);
                        var p2 = zGetNodePosition(rod.getParent(), cc.p(cc.winSize.width, 750 / 3));
                        var distance = cc.pDistance(rod.getPosition(), p2);
                        var time = distance / cc.winSize.width * 1.2;
                        hook.getBody().activate();
                        rod.runAction(cc.sequence(
                            cc.moveTo(time, p2),
                            cc.delayTime(1),
                            cc.callFunc(function () {
                                // 移除鱼钩上的鱼
                                fishNode.removeFromParent(true);
                                fishCount--;
                                zResumeTargetEvent(rod, true);
                                // 鱼篓里加鱼
                                var fish_caught = zNewSpriteCreate(dir, 'fish_caught_' + idt, self.fish_basket);
                                fish_caught.setPosition(cc.pAdd(cc.p(139.77, 236.33), cc.p((Math.random() * 2 - 1) * 30, (Math.random() * 2 - 1) * 30)));
                                fish_caught.runAction(cc.scaleTo(0.5, 0.618).easing(cc.easeOut(2)));
                                ddSound.playEffect('game_dinning_decorate');
                                // 奖励金币
                                var coin_box = Global_DP_Fishing.coin_box;
                                var p1 = cc.pAdd(zGetNodePosition(coin_box, zGetWorldPosition(self.fish_basket)), cc.p(-50, 70));
                                if (!Global_DP_Fishing.gameOver) {
                                    coin_box.runGetCoinAction2(p1, 3);
                                }
                                // coin_box.setCoinNumber(coin_box.getCoinNumber() + 4);

                                if (fishCount == 0) {
                                    // 减到0 奖励时间 重新加鱼
                                    var timeClock = Global_DP_Fishing.timeClock;
                                    timeClock.rewardTime(10);
                                    // timeClock.setCurTime(timeClock.getCurTime() + 30 - self.difficulty * 5);
                                    // 增加难度
                                    self.difficulty += 1;
                                    if (self.difficulty > 4) {
                                        self.difficulty = 4;
                                    }
                                    // 重新加鱼
                                    self.addFish();
                                    self.fish_machine.stopAllActions();
                                    self.fish_machine.runAction(cc.rotateBy(25 - self.difficulty * 5, -360).repeatForever());
                                }
                            }),
                            cc.moveTo(0.8, zGetNodePosition(rod.getParent(), cc.p(cc.winSize.width / 2, cc.winSize.height * 3 / 4)))
                        ));
                        return;
                    }

                    if (name != idt + 'he') {
                        fishNode.scheduleOnce(function () {
                            fishSpine.setAnimation(0, idt + 'zhang', false);
                            fishSpine.setTimeScale(1 + (Math.random() - 0.5) / 3 - 0.1 * self.difficulty);
                        }, 1 + ((Math.random() - 0.5) / 2) + 0.1 * self.difficulty);
                    }
                });
            })(fishNode, fishSpine, idt);
        }
    },
    addTimeClock: function () {
        var self = this;
        var timeTotal = 90;
        var time_out_cb = function () {
            // zLog('time out');
            Global_DP_Fishing.gameOver = true;
            var fish_rod = Global_DP_Fishing.fish_rod;
            fish_rod.stopAllActions();
            zPauseTargetEvent(fish_rod, true);
            zAddAwardCoinLayer(self, Global_DP_Fishing.score);
            umengEvent('GAME_SRC_DONE', 'fishing');
        };
        var timeClock = zAddMyTimeClock(this, timeTotal, time_out_cb);
        timeClock.setLocalZOrder(50);
        Global_DP_Fishing.timeClock = timeClock;

        var count_down_0_cb = function () {
            var fish_rod = Global_DP_Fishing.fish_rod;
            var fish_hook = Global_DP_Fishing.fish_hook;
            zResumeTargetEvent(fish_rod, true);
            timeClock.startTiming();
            fish_hook.getBody().activate();
            fish_rod.runAction(cc.moveBy(0.4, -300, 0).easing(cc.easeIn(2)));

            ddSound.stopAll();
            ddSound.playEffect('game_fishing_little_friend_fishing');
        };
        this.count_down = zAddCountDown(this, 1, 3, count_down_0_cb);
    },
    addCoinBox: function () {
        var coin_box = zAddCoinBox(this);
        coin_box.addScore = function (score) {
            if (Global_DP_Fishing.gameOver) {
                return;
            }
            coin_box.setCoinNumber(coin_box.getCoinNumber() + score);
            Global_DP_Fishing.score += score;
        };
        Global_DP_Fishing.coin_box = coin_box;
    }
});

var DP_FishingScene = cc.Scene.extend({
    ctor: function () {
        this._super();

        var layer = new DP_FishingLayer();
        this.addChild(layer);

        zAddKeyboardBackEventCreate(this);
    }
});